/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UPBUF_H
#define UPBUF_H
#include <cstddef>

// std::size_t e generalmente equvalente a unsigned int
#define MIN(A,B) ( ( (A) <= (B) ) ? (A) : (B) )

template<typename _Tp>
class UpBuf {
public:
	UpBuf();
	UpBuf(std::size_t count);
	UpBuf(std::size_t count, const _Tp& elem);
	UpBuf(const UpBuf& buf);
	~UpBuf();

	_Tp* alloc(std::size_t count);
	_Tp* alloc(std::size_t count,const _Tp& elem);
	_Tp* resize(std::size_t count);
	_Tp* resize(std::size_t count,const _Tp& elem);
	void clear();
	_Tp* begin();
	_Tp* end();
	_Tp* find(const _Tp& elem);
	std::size_t size();
	std::size_t memsize();
	_Tp& operator[](std::size_t pos);
	UpBuf& operator=(const UpBuf& buf);
	std::size_t write(_Tp* from, std::size_t count);
	std::size_t read(const _Tp* from, std::size_t count);
	_Tp* readForce(const _Tp* from, std::size_t count); // resizes if necessary


private:
	_Tp* _begin;
	_Tp* _end;
	std::size_t _size;
};


template<typename _Tp>
inline UpBuf<_Tp>::UpBuf() : _begin(NULL), _end(NULL), _size(0) {
}

template<typename _Tp>
inline UpBuf<_Tp>::~UpBuf() {
	delete[] _begin;
}

template<typename _Tp>
inline UpBuf<_Tp>::UpBuf(std::size_t count) : _size(count) {
	_begin = new _Tp[count];
	_end = _begin + count;
}

template<typename _Tp>
inline UpBuf<_Tp>::UpBuf(std::size_t count,const _Tp& elem) : _size(count) {
	_begin = new _Tp[count];
	_end = _begin + count;
	for(_Tp* _p=_begin; _p<_end; _p++) *_p=elem;
}

template<typename _Tp>
inline UpBuf<_Tp>::UpBuf(const UpBuf<_Tp>& buf) {
	_begin = new _Tp[buf._size];
	_size = buf._size;
	_end = _begin + _size;
	_Tp* _to = _begin;
	_Tp* _from = buf._begin;
	while( _to < _end ) *_to++ = *_from++;
	return *this;
}

template<typename _Tp>
inline void UpBuf<_Tp>::clear() {
	if( _begin!=NULL ) {
		delete[] _begin;
		_begin = _end = NULL;
		_size = 0;
	}
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::alloc(std::size_t count) {
	delete[] _begin;
	_begin = new _Tp[count];
	_end   = _begin + count;
	_size  = count;
	return _begin;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::alloc(std::size_t count,const _Tp& elem) {
	delete[] _begin;
	_begin = new _Tp[count];
	_end   = _begin + count;
	_size  = count;
	for(_Tp* _p=_begin; _p<_end; _p++) *_p=elem;
	return _begin;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::resize(std::size_t count) {
	if( count == _size ) return _begin;
	_Tp* _newbegin = new _Tp[count];
	_Tp* _to = _newbegin;
	_Tp* _from = _begin;
	_end = _begin + MIN(count,_size);
	while( _from < _end ) *_to++ = *_from++;
	delete[] _begin;
	_begin = _newbegin;
	_end = _begin + count;
	_size = count;
	return _begin;
}
	
template<typename _Tp>
inline _Tp* UpBuf<_Tp>::resize(std::size_t count, const _Tp& elem) {
	if( count == _size ) return _begin;
	_Tp* _newbegin = new _Tp[count];
	_Tp* _to = _newbegin;
	_Tp* _from = _begin;
	_end = _begin + MIN(count,_size);
	while( _from < _end ) *_to++ = *_from++;
	_end = _newbegin + count;
	while( _to < _end ) *_to++ = elem;
	delete[] _begin;
	_begin = _newbegin;
	_size = count;
	return _begin;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::begin() {
	return _begin;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::end() {
	return _end;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::find(const _Tp& elem) {
	for(_Tp* _p=_begin; _p<_end; _p++) if( *_p == elem ) return _p;
	return NULL;
}

template<typename _Tp>
inline std::size_t UpBuf<_Tp>::size() {
	return _size;
}

template<typename _Tp>
inline std::size_t UpBuf<_Tp>::memsize() {
	return _size * sizeof(_Tp);
}

template<typename _Tp>
inline _Tp& UpBuf<_Tp>::operator[](std::size_t pos) {
	return _begin[pos];
}

template<typename _Tp>
inline UpBuf<_Tp>& UpBuf<_Tp>::operator=(const UpBuf& buf) {
	delete[] _begin;
	_begin = new _Tp[buf._size];
	_size = buf._size;
	_end = _begin + _size;
	_Tp* _to = _begin;
	_Tp* _from = buf._begin;
	while( _to < _end ) *_to++ = *_from++;
	return *this;
}

template<typename _Tp>
inline std::size_t UpBuf<_Tp>::write(_Tp* to, std::size_t count) {
	count = MIN(_size,count);
	_Tp* __end  = _begin + count;
	_Tp* __from = _begin;
	while( __from < __end ) *to++ = *__from++;
	return count;
}

template<typename _Tp>
inline std::size_t UpBuf<_Tp>::read(const _Tp* from, std::size_t count) {
	count = MIN(_size,count);
	const _Tp* __end = _begin + count;
	_Tp* __to  = _begin;
	while( __to < __end ) *__to++ = *from++;
	return count;
}

template<typename _Tp>
inline _Tp* UpBuf<_Tp>::readForce(const _Tp* from, std::size_t count) {
	if(_size<count) resize(count);
	const _Tp* __end = _begin + count;
	_Tp* __to  = _begin;
	while( __to < __end ) *__to++ = *from++;
	return _begin;
}

#endif
