/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPINET_INL
#define UPINET_INL

inline
UpAddress& UpAddress::setf(const std::string& s,int fmly)
{ return setf(s.c_str(),fmly); }

inline
UpAddress& UpAddress::operator=(const std::string& s)
{ return (*this)=s.c_str(); }

inline
void*			UpAddress::ptr()
{ return (void*)&upAddr; }

inline
const void*	UpAddress::ptr() const
{ return (const void*)&upAddr; }

inline
int UpAddress::family() const
{ return addrType; }

inline
bool UpAddress::operator==(const UpSockAddr& usa) const
{ return usa==*this; }

inline
bool UpAddress::operator==(const UpHost& uh) const
{ return uh==*this; }

inline
bool UpHost::status() const
{ return lookupOk; }

inline
int UpHost::errNumber() const
{ return lookupErrno; }

inline
bool UpSockAddr::hostPort(const std::string& hostAddr,unsigned short port)
{ return hostPort(hostAddr.c_str(),port); }

inline
bool UpSockAddr::hostPort(const std::string& hostAddr)
{ return hostPort(hostAddr.c_str()); }

inline
sockaddr* UpSockAddr::ptr()
{ return sa.a; }

inline
const sockaddr* UpSockAddr::ptr() const
{ return sa.a; }

inline
const char* UpSockAddr::c_str() const
{
	return string().c_str();
}

inline
const char* UpSockAddr::c_str_2dots() const
{ return string_2dots().c_str(); }

inline
const char* UpSockAddr::c_str_ip() const
{ return string_ip().c_str(); }

inline
bool UpSockAddr::set(const std::string& s,unsigned short port,int fmly)
{ return set(s.c_str(),port,fmly); }

inline
bool UpSockAddr::operator==(const UpHost& uh) const
{ return uh==*this; }

inline
bool UpSockAddr::operator>(const UpSockAddr& usa) const
{ return usa<(*this); }

inline
const UpSockAddr& UpSocket::getLocalSockAddr()
{ return localSockAddr; }

inline
const UpSockAddr& UpSocket::getRemoteSockAddr()
{ return remoteSockAddr; }

inline
bool UpSocket::bindInterface(	const std::string& interf,
										unsigned short localPort,int family)
{ return bindInterface(interf.c_str(),localPort,family); }


inline
bool UpAddress::operator>(const UpAddress& upa) const
{ return upa<(*this); }

inline
bool UpAddress::operator<=(const UpAddress& upa) const
{ return !((*this)>upa); }

inline
bool UpAddress::operator>=(const UpAddress& upa) const
{ return !((*this)<upa); }

inline
bool UpAddress::operator!=(const UpAddress& upa) const
{ return !((*this)==upa); }

inline
UpAddress UpAddress::operator~() const
{ UpAddress ret(*this); ret.negate(); return ret; }

inline
UpAddress UpAddress::operator+(const UpAddress& upa) const
{ UpAddress ret(*this); return (ret+=upa); }

inline
UpAddress UpAddress::operator&(const UpAddress& upa) const
{ UpAddress ret(*this); return (ret&=upa); }

inline
UpNet& UpNet::operator=(const UpNet& net)
{ set(net); return *this; }

inline
UpNet& UpNet::operator=(const std::string& net)
{ set(net); return *this; }

inline
unsigned int UpNet::getMaskLen() const
{ return m.maskBitsLen(); }

inline
const UpAddress& UpNet::getBase() const
{ return ip; }

inline
const UpAddress& UpNet::getMask() const
{ return m; }

inline
int UpNet::family() const
{ return ip.addrType; }

inline
UpNetRelation UpNet::check(const UpSockAddr& addr) const
{ return check(UpAddress(addr)); }

inline
UpNetRelation UpNet::check(const in_addr& addr) const
{ return check(UpAddress(addr)); }

inline
UpNetRelation UpNet::check(const in6_addr& addr) const
{ return check(UpAddress(addr)); }

inline
bool UpNet::includes(const UpNet& net) const
{ return (ip.family()!=net.family() or m>net.m) ? false : ((net.ip&m)==ip); }

inline
bool UpNet::includes(const UpAddress& addr) const
{ return (ip.family()!=addr.family())  ? false : ((addr&m)==ip); }

inline
bool UpNet::includes(const UpSockAddr& addr) const
{ return includes(UpAddress(addr)); }

inline
bool UpNet::includes(const in_addr& addr) const
{ return includes(UpAddress(addr)); }

inline
bool UpNet::includes(const in6_addr& addr) const
{ return includes(UpAddress(addr)); }

inline 
const std::string& UpNet::relationString(UpNetRelation r)
{ if(r>UPNR_INVALID_RELATION) r=UPNR_INVALID_RELATION; return relString[r]; }


#endif
