/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPTIMER_H
#define UPTIMER_H
#include <UpTools/UpLog.h>
#include <UpTools/UpTimeVal.h>


class UpTimerManager;

class UpTimer {
protected:
	UpTimeVal when;
	UpTimeVal interval;
	unsigned int count;
	// para ser usado dentro del action
	void repeat(const UpTimeVal& _interval, unsigned int _repeatCount);
private:
	bool adminAction(const UpTimeVal& _now);
public:
	friend class UpTimerManager;
	// constructor one shot
	UpTimer(const UpTimeVal& _when);
	// constructor con repeticion
	UpTimer(const UpTimeVal& _when,
			  const UpTimeVal& _interval, unsigned int _repeatCount);
	// destructor
	virtual ~UpTimer();
	// operadores
	bool operator<(const UpTimer& t) const;
	bool operator<(const UpTimeVal& t) const;
	bool operator>(const UpTimeVal& t) const;
	// metodo a ser reemplazado en lsa clases que hereden de esta
	virtual bool action(const UpTimeVal& _now); 
		// _now se pasa porque quizas el catchup se ejecuto tarde,
		// y entonces 'when' ya paso
};


////////////////////////////////////////////////////////////////////
// metodos de UpTimer

inline void UpTimer::repeat(const UpTimeVal& _interval, unsigned int _repeatCount)
	{ interval = _interval; count = _repeatCount+1; }

inline bool UpTimer::adminAction(const UpTimeVal& _now) { 
	if( action(_now) && --count ) { 
		when+=interval; 
		return true; 
	}
	return false;
}

inline UpTimer::UpTimer(const UpTimeVal& _when)
	: when(_when), interval(0), count(1) { }

inline UpTimer::UpTimer(const UpTimeVal& _when, const UpTimeVal& _interval,
					unsigned int _repeatCount)
	: when(_when), interval(_interval), count(_repeatCount) { }

inline UpTimer::~UpTimer()
	{}

inline bool UpTimer::operator<(const UpTimer& t) const
	{ return (when<t.when) ? true : ( (when>t.when) ? false : (this<&t) ) ; }

inline bool UpTimer::operator<(const UpTimeVal& t) const
	{ return (when<t); }
	
inline bool UpTimer::operator>(const UpTimeVal& t) const
	{ return (when>t); }

inline bool UpTimer::action(const UpTimeVal& _now)
	{ UPLOG(LOG_DEBUG,"UpTimer: dummy action"); return true; }

#endif
