/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UP_MICRO_TIMER_MANAGER_H
#define UP_MICRO_TIMER_MANAGER_H
#include <UpTools/UpThread.h>
#include <UpTools/UpTimer.h>
#include <ext/slist>
#include <set>

////////////////////////////////////////////////////////////////////7
// default values (TICKINTERVAL=0.1 seconds)
////////////////////////////////////////////////////////////////////7
#define TICKINTERVALSEC  ((time_t)0)
#define TICKINTERVALUSEC ((suseconds_t)100000)

////////////////////////////////////////////////////////////////////7
// conjuntos de UpTimer
////////////////////////////////////////////////////////////////////7

// Operador de orden que considera tiempo y posicion de memoria entre
// punteros.
// (entre UpTimers ya existia, esto es para comparar UpTimers*)
class UpTimerPtrLessThan {
public: bool operator()(const UpTimer* a,const UpTimer* b)
	{ return (*a) < (*b); }
};

// single linked list
typedef __gnu_cxx::slist<UpTimer*>							UpTimerSList;
// set sorted by timer memory address
typedef std::set<UpTimer*>                      UpTimerPtrSet;
// set sorted by timer action time and memory address
typedef std::set<UpTimer*,UpTimerPtrLessThan>   UpTimerSet;


////////////////////////////////////////////////////////////////////7
// Class UpTimerManager
////////////////////////////////////////////////////////////////////7
class UpTimerManager : public UpThread {
private:
   UpMutex      mx;
   UpTimerSList tAddList;
   UpTimerSet   tSet;
   UpTimeVal    tickInterval;
   UpTimeVal    nowTime;
   unsigned int catchUp();

public:
	// constructor
   UpTimerManager()
      : tickInterval(TICKINTERVALSEC,TICKINTERVALUSEC) { nowTime.now(); }

	// destructor
   ~UpTimerManager();

	// setea intervalo entre ejecuciones del catchup 
   void setTickInterval(const UpTimeVal& _tickInterval)
      { tickInterval = _tickInterval; }

   // para obtener la hora con una precision igual al tickInterval.
	// Si hace falta mas precision, usar otra cosa
 	const UpTimeVal& now()
      { return nowTime; }

	time_t now_t()
		{ return nowTime.tv_sec; }

	// agrega timers
   void add(UpTimer* _timer)
      { mx.lock(); tAddList.push_front(_timer); mx.unlock(); }

	// el codigo contenido en run se ejecua en un thread independiente cuando
	// se hace startThread()
   bool startThread()
      { return create(); }

	// en cambio se ejecuta en el thread llamador si se hace run()
   void* run();

};
#endif 
