/* UpTools v8.5
 *
 * Copyright (c) 2005-2010 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpThreadWorkDistributor.h>
#include <UpTools/UpArch.h>

#include <iostream>
#include <unistd.h>

using namespace std;

UpMutex mx;

class WorkTest : public UpWork {
	private:
		unsigned int duration;
	public:
		WorkTest(unsigned int _duration) : duration(_duration) {}
		void action() {
			sleep(duration);
			mx.lock();
			cout<<"                                                 "
					<<"Work "<<PTR2PRINTFCAST(this)<<" is completing on thread id"
					<<(unsigned int)pthread_self()<<endl;
			mx.unlock();
		}
};



int main() {
	UpThreadWorkDistributor ld(3);
	ld.setWorkMemoryLength(10);
	ld.setMaxQueueWait(10.0);
	for(int i=0; i<15; ++i ) {
		time_t t = i<10 ? 2 : 1;
		WorkTest* lt=new WorkTest(t);
		string txt;
		if( ld.processWork(lt) ) {
		  txt <<"Added new work with duration="<<t<<'\n'
				<<"WorkArrivalRate: "<<ld.getWorkArrivalRateUpdated()
				<<" WorkExecutionAvgTime: "<<ld.getWorkExecutionAvgTime();
		} else {
		  txt<<"failed to add work\n"
				<<"WorkArrivalRate: "<<ld.getWorkArrivalRateUpdated()
				<<" WorkExecutionAvgTime: "<<ld.getWorkExecutionAvgTime();
		}
		mx.lock();
			cout<<txt<<endl;
		mx.unlock();
		sleep(i<10 ? 1 : 2);
	}
	for(int i=0; i<5; ++i ) {
		mx.lock();
		cout<<"WorkArrivalRate: "<<ld.getWorkArrivalRateUpdated()
				<<" WorkExecutionAvgTime: "<<ld.getWorkExecutionAvgTime()<<endl;
		mx.unlock();
		sleep(1);
	}
	return 0;
}

